import java.awt.*;
import java.lang.Math.*;

/**
 *Subclase de Forma que permite crear la figura Circulo.
 *<p>
 *Contiene los metodos para modificar los circulos.
 *@author Patricio A. Castillo Pizarro - Eric C. Maldonado Olguin
 *@version 1.2 - April 12th, 2007
 */
public class Circulo extends Forma
{
    /**
     *Radio del Circulo
     */
    private double _radio;
    
    /**
     *Centro del Circulo
     */
    private Point _centro;
    
    /**
     *Es el constructor. Inicializa un nuevo Circulo
     *@param c Punto que define el centro del nuevo circulo
     *@param radio Radio del nuevo circulo
     */
    public Circulo (Point c, double radio)
    {
        _radio = Math.abs (radio);
        _centro = new Point ();
        _centro.setLocation (c.getX (), c.getY ());
    }

    /**
     *Metodo que permite Escalar, es decir, cambia el tamano del Circulo en el factor dado respecto al centro de escalamiento dado
     *@param centro Centro relativo del Escalamiento.
     *@param factor Factor de Escalamiento
     */
    public void escalar (Point centro, double factor)
    {
        _centro.setLocation(factor*(_centro.getX()-centro.getX())+centro.getX(), factor*(_centro.getY()-centro.getY())+centro.getY());
        _radio = factor*_radio;
    }

    /**
     *Metodo que permite Rotar, es decir, gira el Circulo en el angulo dado respecto al centro de rotacion dado
     *@param centro Centro de Rotacion
     *@param deltaAngulo Angulo de Rotacion
     */
    public void rotar (Point centro, double deltaAngulo)
    {
        
        /**Si el centro de rotacion es igual al centro del circulo, el circulo sigue siendo el mismo*/
        if (!(_centro.equals(centro)))
        {
            
            /**La pendiente de la linea que une el centro del circulo con el centro de rotacion*/
            double alpha = Math.atan((_centro.getY()-centro.getY())/(_centro.getX()-centro.getX()));
            
            /**La distancia euclidiana entre el centro del circulo y el centro de rotacion*/
            double distancia = Math.hypot((_centro.getX()-centro.getX()),(_centro.getY()-centro.getY()));
            
            /**Si el centro del circulo esta a la derecha del centro de rotacion, el primero debe moverse segun el valor de la pendiente*/
            if ((_centro.getX()-centro.getX())>0)
            {
                _centro.setLocation(distancia*Math.cos(alpha + Math.toRadians(deltaAngulo))+centro.getX(),distancia*Math.sin(alpha + Math.toRadians(deltaAngulo)) +centro.getY());
            }
            
            /**Si el centro del circulo esta a la izquierda del centro de rotacion, el primero debe moverse segun el valor de la pendiente mas pi*/
            else
            {
                _centro.setLocation(distancia*Math.cos(alpha + Math.PI + Math.toRadians(deltaAngulo)) + centro.getX(),distancia*Math.sin(alpha + Math.PI + Math.toRadians(deltaAngulo))+centro.getY());
            }
        }
    }
    
    /**
     *Metodo que permite Trasladar, es decir, cambia la posicion del Circulo
     *@param deltaX Desplazamiento horizontal
     *@param deltaY Desplazamiento vertical
     */
    public void trasladar (double deltaX, double deltaY)
    {
        _centro.setLocation(_centro.getX() + deltaX, _centro.getY() + deltaY);
    }
    
    /**
     *Metodo que permite verificar la Igualdad de dos objetos, es decir, compara contenidos de dos Circulos
     *@param otherObject Otro objeto Forma
     *@return true Si el contenido de los circulos son iguales y false si son distintos
     */
    public boolean equals (Object otherObject)
    {
        /**verifica que exista el otro objeto*/
        if (otherObject == null) return false;
        
        /**verifica si los objetos son identicos*/
        if (this == otherObject) return true;
        
        /**para ser iguales deben ser de la misma clase*/
        if (getClass() != otherObject.getClass()) return false;
        
        /**se sabe que el otro objeto es un Circulo, por lo que se comparan los atributos*/
        Circulo other = (Circulo) otherObject;
        return _centro.equals (other._centro) && _radio == other._radio;
    }
    
    /**
     *Metodo que permite Clonar, es decir, duplica un Circulo
     *@return la referencia del Circulo clonado
     */
    public  Circulo clone ()
    {
        Circulo cloned = new Circulo (_centro,_radio);
        return cloned;
    }
    
    /**
     *Metodo que permite mostrar en consola los atributos del Circulo
     *@return Atributos del Circulo
     */
    public String toString ()
    {
        return getClass().getName() + ": Centro("+ _centro.getX()+ "," + _centro.getY()+ ") - Radio=" + _radio + "." ;
    }
    
    /**
     *Metodo que permite entregar el valor del parametro Centro del Circulo
     *@return Entrega el valor del Centro del Circulo
     */
    public Point getCentro ()
    {
        return _centro;
    }
    
    /**
     *Metodo que permite entregar el valor del parametro Radio del Circulo
     *@return Entrega el valor del Radio del Circulo
     */
    public double getRadio ()
    {
        return _radio;
    }
}
/**Fin subclase Circulo
 *Principal dificultad: Definir el algoritmo del metodo de rotacion
 *Solucion: Metodo mas simple, paso a paso
 */