import java.awt.*;

/**
 *Clase base para definir distintas formas en el plano cartesiano.
 *<p>
 *Contiene los metodos para modificar las formas.
 *@author Patricio A. Castillo Pizarro - Eric C. Maldonado Olguin
 *@version 1.1 - April 12th, 2007
 */
public abstract class Forma
{
    /**Metodo que permite Escalar la forma, es decir, cambia sus proporciones respecto a un punto central en el factor dado.
     *@param centro Punto de referencia para el escalamiento.
     *@param factor Factor de magnitud para el escalamiento. Si factor > 1 el escalamiento es amplificacion. Si factor < 1 el escalamiento es reduccion.
     */
    public abstract void escalar(Point centro, double factor);
    
    /**Metodo que permite Rotar la forma, es decir, la gira respecto a un punto central en el angulo dado.
     *@param centro Punto de referencia para la rotacion.
     *@param deltaAngulo Angulo en radianes para la rotacion.
     */
    public abstract void rotar(Point centro, double deltaAngulo);
    
    /**Metodo que permite Trasladar la forma, es decir, cambia su ubicacion desplazandola en el plano cartesiano.
     *@param deltaX Medida horizontal de la traslacion.
     *@param deltaY Medida vertical de la traslacion.
     */
    public abstract void trasladar(double deltaX, double deltaY);
}
/**Fin superclase Forma*/