import java.awt.*;
import java.lang.Math.*;

/**
 *Subclase de Forma que permite crear la figura Rectangulo.
 *<p>
 *Contiene los metodos para modificar los rectangulos.
 *@author Patricio A. Castillo Pizarro - Eric C. Maldonado Olguin
 *@version 1.2 - April 12th, 2007
 */
public class Rectangulo extends Forma
{
    /**
     *Puntos que definen al rectangulo y el centro de este
     */
    private Point _a, _b, _centro;
    
    /**
     *Valores utiles de Ancho, Alto y el Angulo de Inclinacion del Rectangulo
     */
    private double _ancho, _alto, _angulo;
    
    /**
     *Es el constructor. Inicializa un nuevo Rectangulo
     *@param a Primer punto que define el nuevo rectangulo
     *@param b Segundo punto que define el nuevo rectangulo
     */
    public Rectangulo (Point a, Point b)
    {
        _a = new Point ();
        _a.setLocation (a.getX (),a.getY ());
        _b = new Point ();
        _b.setLocation (b.getX (), b.getY ());
        _centro = new Point ();
        _centro.setLocation ((_a.getX() + _b.getX())/2, (_a.getY() + _b.getY())/2);
        _angulo = 0;
        _ancho = Math.abs (_a.getX() - _b.getX());
        _alto = Math.abs (_a.getY() - _b.getY());
    }
    
    /**
     *Metodo que permite Escalar, es decir, cambia el tamano del Rectangulo
     *@param centro Centro relativo del Escalamiento
     *@param factor Factor de Escalamiento
     */
    public void escalar (Point centro, double factor)
    {
        _centro.setLocation(factor*(_centro.getX()-centro.getX())+centro.getX(), factor*(_centro.getY()-centro.getY())+centro.getY());
        _a.setLocation(factor*(_a.getX()-centro.getX())+centro.getX(), factor*(_a.getY()-centro.getY())+centro.getY());
        _b.setLocation(factor*(_b.getX()-centro.getX())+centro.getX(), factor*(_b.getY()-centro.getY())+centro.getY());
        _ancho = _ancho*factor;
        _alto = _alto*factor;
    }
    
    /**
     *Metodo que permite Rotar, es decir, gira el Rectangulo
     *@param centro Centro de Rotacion
     *@param deltaAngulo Angulo de Rotacion
     */
    public void rotar (Point centro, double deltaAngulo)
    {
        /**Si el centro de rotacion es igual al centro del rectangulo, el centro del rectangulo no cambia*/
        if (!(_centro.equals(centro)))
        {
            
            /**La pendiente de la linea que une el centro del rectangulo con el centro de rotacion*/
            double alpha = Math.atan((_centro.getY()-centro.getY())/(_centro.getX()-centro.getX()));
            
            /**La distancia euclidiana entre el centro del rectangulo con el centro de rotacion*/
            double distancia = Math.hypot((_centro.getX()-centro.getX()),(_centro.getY()-centro.getY()));
            
            /**Si el centro del rectangulo esta a la derecha del centro de rotacion, el primero debe moverse segun el valor de la pendiente*/
            if ((_centro.getX()-centro.getX())>0)
            {
                _centro.setLocation(distancia*Math.cos(alpha + Math.toRadians(deltaAngulo)) + centro.getX(), distancia*Math.sin(alpha + Math.toRadians(deltaAngulo)) + centro.getY());
            }
            
            /**Si el centro del rectangulo esta a la izquierda del centro de rotacion, el primero debe moverse segun el valor de la pendiente mas pi*/
            else
            {
                _centro.setLocation(distancia*Math.cos(alpha + Math.PI + Math.toRadians(deltaAngulo)) + centro.getX(), distancia*Math.sin(alpha + Math.PI + Math.toRadians(deltaAngulo)) + centro.getY());
            }
        }
        
        /**Si el centro de rotacion es igual al punto _a, el punto _a no cambia*/
        if (!(_a.equals(centro)))
        {
            
            /**La pendiente de la linea que une el punto _a con el centro de rotacion*/
            double betha = Math.atan((_a.getY()-centro.getY())/(_a.getX()-centro.getX()));
            
            /**La distancia euclidiana entre el punto _a y el centro de rotacion*/
            double d1 = Math.hypot((_a.getX()-centro.getX()),(_a.getY()-centro.getY()));
            
            /**Si el punto _a esta a la derecha del centro de rotacion, el primero debe moverse segun el valor de la pendiente*/
            if ((_a.getX()-centro.getX())>0)
            {
                _a.setLocation(d1*Math.cos(betha + Math.toRadians(deltaAngulo)) + centro.getX(), d1*Math.sin(betha + Math.toRadians(deltaAngulo)) + centro.getY());
            }
            
            /**Si el punto _a esta a la izquierda del centro de rotacion, el primero debe moverse segun el valor de la pendiente mas pi*/
            else
            {
                _a.setLocation(d1*Math.cos(betha + Math.PI + Math.toRadians(deltaAngulo)) + centro.getX(),d1*Math.sin(betha + Math.PI + Math.toRadians(deltaAngulo)) + centro.getY());
            }
        }
        
        /**Si el centro de rotacion es igual al punto _b, el punto _b no cambia*/
        if (!(_b.equals(centro)))
        {
            
            /**La pendiente de la linea que une el punto _b con el centro de rotacion*/
            double gamma = Math.atan((_b.getY()-centro.getY())/(_b.getX()-centro.getX()));
            
            /**La distancia euclidiana entre el punto _b y el centro de rotacion*/
            double d2 = Math.hypot((_b.getX()-centro.getX()),(_b.getY()-centro.getY()));
            
            /**Si el punto _b esta a la derecha del centro de rotacion, el primero debe moverse segun el valor de la pendiente*/
            if ((_b.getX()-centro.getX())>0)
            {
                _b.setLocation(d2*Math.cos(gamma + Math.toRadians(deltaAngulo)) + centro.getX(),d2*Math.sin(gamma + Math.toRadians(deltaAngulo)) + centro.getY());
            }
            
            /**Si el punto _b esta a la izquierda del centro de rotacion, el primero debe moverse segun el valor de la pendiente mas pi*/
            else
            {
                _b.setLocation(d2*Math.cos(gamma + Math.PI + Math.toRadians(deltaAngulo)) + centro.getX(),d2*Math.sin(gamma + Math.PI + Math.toRadians(deltaAngulo)) + centro.getY());
            }
        }
        
        /**El nuevo angulo de inclinacion se determina segun el angulo de inclinacion*/
        _angulo = (_angulo + Math.toRadians(deltaAngulo))%(2*Math.PI);
    }
    
    /**
     *Metodo que permite Trasladar, es decir, cambia la posicion del Rectangulo
     *@param deltaX Desplazamiento horizontal
     *@param deltaY Desplazamiento vertical
     */
    public void trasladar (double deltaX, double deltaY)
    {
        _centro.setLocation(_centro.getX() + deltaX, _centro.getY() + deltaY);
        _a.setLocation(_a.getX() + deltaX, _a.getY() + deltaY);
        _b.setLocation(_b.getX() + deltaX, _b.getY() + deltaY);
    }
    
    /**
     *Metodo que permite verificar la Igualdad de dos objetos, es decir, compara contenidos de dos Rectangulos
     *@param otherObject Otro objeto Forma
     *@return true Si el contenido de los rectangulos son iguales y false si son distintos
     */
    public boolean equals (Object otherObject)
    {
        /**verifica que exista el otro objeto*/
        if (otherObject == null) return false;
        
        /**verifica si los objetos son identicos*/
        if (this == otherObject) return true;
        
        /**para ser iguales deben ser de la misma clase*/
        if (getClass() != otherObject.getClass()) return false;
        
        /**se sabe que el otro objeto es un Rectangulo, por los que se comparan los atributos*/
        Rectangulo other = (Rectangulo) otherObject;
        return _ancho == other._ancho && _alto == other._alto && _centro.equals (other._centro) && _a.equals(other._a) && _b.equals(other._b);
    }
    
    /**
     *Metodo que permite Clonar, es decir, duplica un Rectangulo
     *@return la referencia del Rectangulo clonado
     */
    public Rectangulo clone ()
    {
        Rectangulo cloned = new Rectangulo (_a, _b);
        cloned._angulo = this._angulo;
        cloned._ancho = this._ancho;
        cloned._alto = this._alto;
        return cloned;
    }
    
    /**
     *Metodo que permite mostrar en consola los atributos del Rectangulo
     *@return Atributos del Rectangulo
     */
    public String toString ()
    {
        return getClass().getName() + ": Centro("+ _centro.getX()+ "," + _centro.getY()+ ") - Ancho=" + _ancho + " - Alto=" + _alto + " - Inclinacion=" + Math.toDegrees(_angulo) + " grados.";
    }
    
    /**
     *Metodo que permite entregar el valor del parametro Centro del Rectangulo
     *@return Entrega el valor del Centro del Rectangulo
     */
    public Point getCentro ()
    {
        return _centro;
    }
    
    /**
     *Metodo que permite entregar el valor del parametro _a del Rectangulo
     *@return Entrega el valor del punto _a del Rectangulo
     */
    public Point getA ()
    {
        return _a;
    }
    
    /**
     *Metodo que permite entregar el valor del parametro _b del Rectangulo
     *@return Entrega el valor del punto _b del Rectangulo
     */
    public Point getB ()
    {
        return _b;
    }
    
    /**
     *Metodo que permite entregar el valor del parametro Ancho del Rectangulo
     *@return Entrega el valor del Ancho del Rectangulo
     */
    public double getAncho ()
    {
        return _ancho;
    }
    
    /**
     *Metodo que permite entregar el valor del parametro Alto del Rectangulo
     *@return Entrega el valor del Alto del Rectangulo
     */
    public double getAlto ()
    {
        return _alto;
    }
    
    /**
     *Metodo que permite entregar el valor del parametro Angulo del Rectangulo
     *@return Entrega el valor del Angulo de Inclinacion del Rectangulo
     */
    public double getAngulo ()
    {
        return _angulo;
    }
}
/**fin subclase Rectangulo
*Principal dificultad: Comportamiento de la definicion del centro del rectangulo, dado que los puntos que lo definen son creados con valores de doble presicion, pero se almacenan aproximandose al valor entero mas cercano
*Solucion: Ingresar datos enteros. Se podria haber definido una nueva clase de punto con doble presicion
 */